Exception: Dewasm::Pozeiden::Error

Inherits:
StandardError
  • Object
show all
Defined in:
lib/dewasm/pozeiden.rb

Overview

Raised when pozeiden returns a Zig error, and for input this gem rejects before calling into the WebAssembly module.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(message, zig_error: nil) ⇒ Error

Returns a new instance of Error.



16
17
18
19
# File 'lib/dewasm/pozeiden.rb', line 16

def initialize(message, zig_error: nil)
  super(message)
  @zig_error = zig_error
end

Instance Attribute Details

#zig_errorObject (readonly)

The @errorName reported by pozeiden, or nil for a local rejection.



14
15
16
# File 'lib/dewasm/pozeiden.rb', line 14

def zig_error
  @zig_error
end